package com.mhsoftware.jonnie;
/*

    MH Software, Inc.
    5023 W 120th Ave, #311
    Broomfield, CO 80020

    Voice: 303 438 9585
    Web: http://www.mhsoftware.com/

    Copyright 2008, MH Software, Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.nio.ByteBuffer;
import java.nio.LongBuffer;
import java.util.Random;

/**
 * This class takes a buffer and prepares it for use by filling 
 * it with a specific value, or filling it with a random value. 
 *  
 * @author George Sexton 
 */
public class BlockDataGenerator {

    private BlockDataGenerator(){
    }

    /**
     * Fill the buffer with a specified long value. 
     *  
     * @param buf The Buffer to fill. 
     * @param lValue the long value to use. 
     */
    public static void fillBuffer(LongBuffer buf,long lValue){
	final int iCapacity=buf.capacity();
        for (int i=0; i < iCapacity; i++) {
            buf.put(lValue);
        }
    }

    /**
     * Fill the buffer with a collection of random/pseudo random 
     * data. We're not shooting to be cryptographically correct 
     * here. 
     *  
     * @param buf The buffer to fill. 
     */
    public static void fillBufferRandom(ByteBuffer buf){
        fillBufferRandom(buf,new Random());
    }

    public static void fillBufferRandom(ByteBuffer buf,Random prng){
        LongBuffer lb=buf.asLongBuffer();
	final int iLimit=lb.capacity();
        for (int i=0; i < iLimit; i++) {
            lb.put(prng.nextLong());
        }

    }
}
