package com.mhsoftware.jonnie;
/*
    MH Software, Inc.
    5023 W 120th Ave, #311
    Broomfield, CO 80020

    Voice: 303 438 9585
    Web: http://www.mhsoftware.com/

    Copyright 2008, MH Software, Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** 
 * This interface defines an event listener. The listener is
 * responsible for recording events, providing feedback to
 * operators, and recording results.
 * 
 * @author George Sexton
 * 
 */
interface TestEventListener {
    /** 
     * Method called prior to test begin.
     * 
     * @param tst
     */
    void testBegin(JonnieTest tst);

    /** 
     * Method fired when a test ends. This may fire at the normal 
     * completion of the test, or if the test is interrupted. 
     * 
     * @param tst
     */
    void testEnd(JonnieTest tst);

    /** 
     * Method called when a test is completed. At the point this is 
     * called, the test must have it's results readily available via 
     * the getResults() method. 
     * 
     * @param tst
     */
    void testComplete(JonnieTest tst);


    /** 
     * This method shold be fired when an exception occurs during a 
     * test. 
     * 
     * @param e
     * @param tst
     */
    void testExceptionEvent(Exception e,JonnieTest tst);

	/** 
	 * Log a generic message.
	 * 
	 * @param sMessage
	 */
	void log(String sMessage);

    /** 
     * Return true if an exception was reported via 
     * testExceptionEvent. 
     * 
     * @return boolean
     */
    boolean wasExceptionThrown();

}
