package com.mhsoftware.jonnie;
/*
    MH Software, Inc.
    5023 W 120th Ave, #311
    Broomfield, CO 80020

    Voice: 303 438 9585
    Web: http://www.mhsoftware.com/

    Copyright 2008, MH Software, Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** 
 * A simple implementation of TestEventListener that just echoes
 * all messages to stdout.
 *  
 * Output to stdout is synchronized so that results from 
 * different threads won't become intermingled. 
 *  
 * @author George Sexton
 * 
 */
public class TestEventListenerImpl implements TestEventListener {

    private boolean bException=false;

    /**
     * Method called prior to test begin.
     *
     * @param tst
     */
    public void testBegin(JonnieTest tst) {
        synchronized (this){
			log("Beginning test with "+tst.getClass().getName());
        }
    }
    /**
     * Method called when a test is completed. At the point this is
     * called, the test must have it's results readily available via
     * the getResults() method.
     *
     * @param tst
     */
    public void testComplete(JonnieTest tst) {
        synchronized (this){
			log("Test "+tst.getClass().getName()+" completed.");
            TestResult[] aResults=tst.getResults();
            for (int i=0; aResults!=null && i<aResults.length; i++) {
				log(aResults[i].getName()+"\t\t\t"+aResults[i].getValue()+"\t"+aResults[i].getUnits());
            }
        }
    }
    /**
     * Method fired when a test ends. This may fire at the normal
     * completion of the test, or if the test is interrupted.
     *
     * @param tst
     */
    public void testEnd(JonnieTest tst) {
        synchronized (this){
			log("Ending test for "+tst.getClass().getName());
        }
    }

    /** 
     * Report an exception that occurred during the test.
     * 
     * @param e The Exception that was thrown.
     * @param tst A reference to the test object.
     */
    public void testExceptionEvent(Exception e,JonnieTest tst){
        bException=true;
		synchronized (this){
			log(tst.getClass().getName()+" threw Exception "+e);
		}
    }

    /** 
     * Return true if any Test using this listener reported a 
     * testExceptionEvent. 
     * 
     * 
     * @return boolean
     */
    public boolean wasExceptionThrown(){
        return bException;
    }

	public void log(String sMessage){
           System.out.println("Thread ["+Thread.currentThread().getName()+"] "+sMessage);
	}
}
