package com.mhsoftware.jonnie;
/*
    MH Software, Inc.
    5023 W 120th Ave, #311
    Broomfield, CO 80020

    Voice: 303 438 9585
    Web: http://www.mhsoftware.com/

    Copyright 2008, MH Software, Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.io.CharArrayReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.*;
import org.xml.sax.InputSource;


/**
 * This class serves as a base class for the XML Input and pickers. It provides
 * convenience methods.
 *
 * @author George Sexton
 *
 * @since 3.3.0
 */
public class XMLBase {

    private static final boolean DEBUG=false;

    public static String readXMLFile(String sName){
        StringBuffer sb=new StringBuffer();
        try {
            
        FileInputStream fis=new FileInputStream(sName);
        InputStreamReader isr=new InputStreamReader(fis);
        char[] buf=new char[1024];
        
        int iCount;
        while ((iCount=isr.read(buf,0,buf.length))>0) {
            sb.append(buf,0,iCount);
        }
        isr.close();
        } catch (IOException ioe) {
            System.out.println(ioe);
        }

        return sb.toString();
    }

    /**
     * Parse the XML Document, returning the Document Element.
     *
     * @param sXML the XML String to Parse
     */
    public static Node parseXMLDocument(String sXML){
		if (sXML==null || sXML.length()==0) {
			throw new IllegalArgumentException("Null or empty XML Document sent to parseXMLDocument()!");
		}
        Node doc=null;
        try {
            DocumentBuilderFactory dbf=DocumentBuilderFactory.newInstance();
            dbf.setValidating(false);
            DocumentBuilder db=dbf.newDocumentBuilder();
            InputSource is=new InputSource(new CharArrayReader(sXML.toCharArray()));
            doc=db.parse(is).getDocumentElement();
        } catch (javax.xml.parsers.ParserConfigurationException e) {
            System.err.println("Parsing Document! - "+e);
        } catch (org.xml.sax.SAXException se) {
            System.err.println("Parsing Document - "+se);
        } catch (java.io.IOException ioe) {
            System.err.println("Who knows why - "+ioe);
        }

        return doc;
    }

    /**
     * Return a specific named node from the parent.
     *
     * @param nodeParent The node containing the desired node.
     * @param elementName The node name.
     *
     * @return The named node, or null if not found.
     */
    public static Node getElementNode(Node nodeParent,String elementName){
        if (DEBUG) {
            System.out.println("getElementNode(<nodeParent="+nodeParent.getNodeName()+">,\""+elementName+"\");");
        }
        Node n=nodeParent.getFirstChild();
        while (n!=null) {
            if (DEBUG) {
                System.out.println("Current Node - Type="+n.getNodeType()+" Name=["+n.getNodeName()+"]");
            }

            if (n.getNodeType()==Node.ELEMENT_NODE && elementName.equals(n.getNodeName())) {
                if (DEBUG) {
                    System.out.println("Returning Current Node - Type="+n.getNodeType()+" Name=["+n.getNodeName()+"]");
                }
                return n;
            }
            n=n.getNextSibling();
        }
        return null;
    }

    /**
     * Return an array of element nodes that have the specified name.
     * <BR><BR>
     * Say for example, you have a FieldSet node. You can use this
     * method to retrieve the Field nodes that comprise that set.
     *
     * @param nodeParent The parent node containing the elements.
     * @param elementName The name of the elements to return.
     *
     * @return An array of node elements. If no elements are found,
     * returns null.
     */
    public static Node[] getElementNodes(Node nodeParent, String elementName){
        ArrayList<Node> al=new ArrayList<Node>(32);
        Node n=nodeParent.getFirstChild();
        while (n!=null) {
            if (n.getNodeType()==Node.ELEMENT_NODE && elementName.equals(n.getNodeName())) {
                al.add(n);
            }
            n=n.getNextSibling();
        }
        Node[] nResult=new Node[al.size()];
        al.toArray(nResult);
        return nResult;

    }

    /**
     * Return the text value associated with this node.
     * @param node The node to get the text from. If node is null, then
     * null is returned.
     */
    public static String getNodeText(Node node){
        if (node==null) {
            return null;
        }
        Node nChild=node.getFirstChild();
        while (nChild!=null) {
            if (nChild.getNodeType()==Node.TEXT_NODE) {
                return nChild.getNodeValue();
            }
            nChild=nChild.getNextSibling();
        }
        return null;
    }

    public static String getElementText(Node nodeParent,final String elementName){
        return getNodeText(getElementNode(nodeParent,elementName));
    }

    /**
     * Get the value of a node attribute.
     *
     * @param node The node to get the attribute from.
     * @param attributeName the name of the attribute to retrieve.
     */
    public static String getAttributeValue(Node node, String attributeName) {
        return getAttributeValue(node,attributeName,null);
    }

    /**
     * Get the value of a node attribute.
     *
     * @param node The node to get the attribute from.
     * @param attributeName the name of the attribute to retrieve.
     * @param defaultValue If the attribute does not exist on the node, the
     * default to return.
     *
     */
    public static String getAttributeValue(Node node, String attributeName, String defaultValue){
        String sValue=defaultValue;
        NamedNodeMap nnm=node.getAttributes();
        if (nnm!=null) {
            Node nAttr=nnm.getNamedItem(attributeName);
            if (nAttr!=null) {
                sValue=nAttr.getNodeValue();
            }
        }
        return sValue;
    }

    /**
     * Because sometimes it's just damn handy to have a blank node.
     * @param name the Name of the node object to create.
     */
    public static Node getEmptyNode(final String name){
        try {
            DocumentBuilderFactory dbf=DocumentBuilderFactory.newInstance();
            dbf.setValidating(false);
            DocumentBuilder db=dbf.newDocumentBuilder();
            Document doc=db.newDocument();
            return doc.createElement(name);

        } catch (javax.xml.parsers.ParserConfigurationException e) {
            //TODO: Write Handler
        }
        return null;
    }

}
